class Helpers:
    @staticmethod
    def isIterable(obj):
        if isinstance(obj, str):
            return False
        try:
            iter(obj)
            obj[0]
            return True
        except Exception:
            return False


def _callableSync(obj, update_dict):
	if getattr(obj, "_callable", False):
		# clumsily allowing args to propagate
		args = getattr(obj, "_args", ())
		kwds = getattr(obj, "_kwargs", dict()).copy()

		shared = set(kwds) & set(update_dict)
		for k in shared:
			kwds[k] = update_dict[k]
		return obj(args, **kwds)
	return obj


def interface_required(func):

	def _wrapper(*args, **kwargs):
		try:
			sys.modules["Inference"]
		except KeyError:
			raise ImportError("Interface module is required for this functionality.")
		return func(*args, **kwargs)

	return _wrapper	# hello



from IPython.display import display
import numpy as np
import plotly.graph_objects as go
import time
import ipywidgets
from IPython.core.displaypub import DisplayPublisher
from IPython.core.interactiveshell import InteractiveShell
from types import ModuleType
import plotly.basedatatypes
import sys
import
class contextAware(type):
	class InterceptingPublisher(DisplayPublisher):
		def publish(self, data, metadata=None, source=None, *, transient=None, update=False):
			if "application/vnd.plotly.v1+json" in data:
				print("show invoked")
				return
			else:
				pass

	@staticmethod
	def isNoteBookEnv():
		
		try:
			ipthn =sys.modules["IPython"]
			ip = getattr(ipthn, "get_ipython")()
			

			if ip is None:
				return False
			return True
		except Exception as e:
			return False

	# Define time processing of environment
	def __new__(cls, name, bases, attrs):
		new_class = super().__new__(cls, name, bases, attrs)
		if contextAware.isNoteBookEnv():
			new_class.shell = InteractiveShell.instance()
			# global instance but assigned to attr for easy retreval
			new_class.shell.display_pub = contextAware.InterceptingPublisher()

		return new_class


def f(*args, **kwargs):
	"""
	Dummy function to test the metaclass.
	"""
	print("display overwrite")


# --- Simplified Demo Class ---
class _test_Plots:

	@staticmethod
	def scatter(x, y):
		return go.Figure(go.Scatter(x=x, y=y, mode="lines+markers"))

	@staticmethod
	def show_scatter(x, y):
		fig = _demo_Plots.scatter(x, y)
		# sendPlotlyFigToGui(fig)
		fig.show()
		return fig

	@staticmethod
	def updateLiveScatter(fig, x_new, y_new):
		if not isinstance(fig, go.Figure):
			fig = go.Figure(fig)
		xs = (
			[*fig.data[0].x, *x_new]
			if isinstance(x_new, (list, tuple))
			else [*fig.data[0].x, x_new]
		)
		ys = (
			[*fig.data[0].y, *y_new]
			if isinstance(y_new, (list, tuple))
			else [*fig.data[0].y, y_new]
		)
		fig.data[0].x = xs
		fig.data[0].y = ys
		# Update data on the figure
		# For FigureWidget, batch_update is better. For go.Figure, direct assignment is fine.
		if isinstance(fig, go.FigureWidget):
			with fig.batch_update():
				fig.data[0].x = xs
				fig.data[0].y = ys
		else:
			fig.data[0].x = xs
			fig.data[0].y = ys

		display(fig)  # Crucial: direct display call
		return fig



# t
# from plot_interceptor import isNoteBookEnv
import plotly.graph_objects as go
from plotly.basedatatypes import BaseFigure
from IPython.display import display

# print(isNoteBookEnv())
import plotly.graph_objects as go
from plotly.basedatatypes import BaseFigure

try:

	shell = InteractiveShell.instance()

	class InterceptingPublisher(DisplayPublisher):
		def publish(self, data, metadata=None, source=None, *, transient=None, update=False):
			if "application/vnd.plotly.v1+json" in data:
				print("show invoked")#hello
				return
			else:
				pass
				# print(f"{data}")
			# return super().publish(
			#     data, metadata, source, transient=transient, update=update
			# )

	shell.display_pub = InterceptingPublisher()

except Exception as e:
	print("Failed to set up custom display publisher:", e)

# # plotly.io.renderers
# fig = go.Figure(data=go.Scatter(x=[1, 2, 3], y=[4, 5, 6]))

NewFigType = type("test_fig", (BaseFigure,), {})


def test_display(self):
	print("displaying")

def test_call(self):	#TEST2
	print("call")# TEST1


NewFigType._ipython_display_ = test_displayƒ
NewFigType.__call__ = test_call

new_fig = NewFigType()
# # object.__setattr__(fig, "enabled", True)

super(NewFigType, new_fig).__setattr__("__call__", test_call)

import inspect

# print(inspect.isclass(fig))
# # print(callable(fig))ƒ
# display(new_fig)
new_fig.show()

# class TestIntercept:
#     from IPython.core.interactiveshell import InteractiveShell
#     from IPython.core.displaypub import DisplayPublisher
